"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postProcessSvg = exports.emitMemberAssociations = exports.emitHeritageClauses = exports.emitSingleEnum = exports.emitSingleType = exports.emitSingleInterface = exports.emitSingleClass = void 0;
const model_1 = require("./model");
const templates_1 = require("./templates");
function emitSingleClass(cls) {
    return templates_1.templates.class(cls.name, cls.properties.map(escapePropertyDetails), cls.methods.map(escapeMethodDetails));
}
exports.emitSingleClass = emitSingleClass;
function emitSingleInterface(int) {
    return templates_1.templates.interface(int.name, int.properties.map(escapePropertyDetails), int.methods.map(escapeMethodDetails));
}
exports.emitSingleInterface = emitSingleInterface;
function emitSingleType(t) {
    return templates_1.templates.type(t.name, t.properties.map(escapePropertyDetails), t.methods.map(escapeMethodDetails));
}
exports.emitSingleType = emitSingleType;
function emitSingleEnum(en) {
    return templates_1.templates.enum(en.name, en.items);
}
exports.emitSingleEnum = emitSingleEnum;
function emitHeritageClauses(heritageClauses) {
    return heritageClauses.map((heritageClause) => {
        if (heritageClause.type === model_1.HeritageClauseType.Extends) {
            return templates_1.templates.extends(heritageClause.clause, heritageClause.className);
        }
        else {
            return templates_1.templates.implements(heritageClause.clause, heritageClause.className);
        }
    });
}
exports.emitHeritageClauses = emitHeritageClauses;
function emitMemberAssociations(associations) {
    return associations ? associations.map(templates_1.templates.memberAssociation) : [];
}
exports.emitMemberAssociations = emitMemberAssociations;
// utility functions
function escapeNomnoml(str) {
    return str.replace(/[|\][\#]/g, '\\$&');
}
function xmlEncode(str) {
    return str
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/"/g, '&quot;')
        .replace(/'/g, '&apos;');
}
function escapeMethodDetails(details) {
    if (details.returnType) {
        details.returnType = escapeNomnoml(details.returnType);
    }
    return details;
}
function escapePropertyDetails(details) {
    if (details.type) {
        details.type = escapeNomnoml(details.type);
    }
    return details;
}
function postProcessSvg(svg, diagramPath, declarations) {
    const classes = {};
    const interfaces = {};
    const enums = {};
    const types = {};
    declarations.map(d => {
        d.classes.forEach(cls => classes[xmlEncode(cls.name)] = cls);
        d.interfaces.forEach(i => interfaces[xmlEncode(i.name)] = i);
        d.enums.forEach(e => enums[xmlEncode(e.name)] = e);
        d.types.forEach(t => types[xmlEncode(t.name)] = t);
    });
    const rx = />(.*)</;
    const arOut = [];
    let regexResult;
    for (let line of svg.split('\n')) {
        line = line.trim();
        if (line.startsWith("<text") && (regexResult = rx.exec(line))) {
            let target = classes[regexResult[1]] || interfaces[regexResult[1]] || enums[regexResult[1]] || types[regexResult[1]];
            if (target) {
                const relPath = target.getRelativeFilePath(diagramPath);
                line = `<a id="${relPath}.${xmlEncode(target.name)}" xlink:href="${relPath}">${line}</a>`;
            }
        }
        arOut.push(line + '\n');
    }
    return arOut.join('');
}
exports.postProcessSvg = postProcessSvg;
//# sourceMappingURL=emitter.js.map