"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createNomnomlSVG = void 0;
const io_1 = require("./io");
const parser_1 = require("./parser/parser");
const emitter_1 = require("./emitter");
const tsuml2_settings_1 = require("./tsuml2-settings");
const chalk_1 = __importDefault(require("chalk"));
const fs = __importStar(require("fs"));
const parser_2 = require("./parser");
function parse(tsConfigPath, pattern) {
    const ast = (0, parser_1.getAst)(tsConfigPath, pattern);
    const files = ast.getSourceFiles();
    // parser
    console.log(chalk_1.default.yellow("parsing source files:"));
    const declarations = files.map(f => {
        const classes = f.getClasses();
        const interfaces = f.getInterfaces();
        const enums = f.getEnums();
        const types = f.getTypeAliases();
        const path = f.getFilePath();
        console.log(chalk_1.default.yellow(path));
        const classDeclarations = classes.map(parser_1.parseClasses);
        const interfaceDeclarations = interfaces.map(parser_1.parseInterfaces);
        return {
            fileName: path,
            classes: classDeclarations,
            interfaces: interfaceDeclarations,
            types: types.map(parser_1.parseTypes).filter(t => t !== undefined),
            enums: enums.map(parser_1.parseEnum),
            heritageClauses: [
                ...classDeclarations.filter(decl => decl.heritageClauses.length > 0).map(decl => decl.heritageClauses),
                ...interfaceDeclarations.filter(decl => decl.heritageClauses.length > 0).map(decl => decl.heritageClauses)
            ]
        };
    });
    if (tsuml2_settings_1.SETTINGS.memberAssociations) {
        (0, parser_2.parseAssociations)(declarations);
    }
    return declarations;
}
function emit(declarations) {
    const entities = declarations.map(d => {
        console.log(chalk_1.default.yellow(d.fileName));
        const classes = d.classes.map((c) => (0, emitter_1.emitSingleClass)(c));
        const interfaces = d.interfaces.map((i) => (0, emitter_1.emitSingleInterface)(i));
        const enums = d.enums.map((i) => (0, emitter_1.emitSingleEnum)(i));
        const types = d.types.map((t) => (0, emitter_1.emitSingleType)(t));
        const heritageClauses = d.heritageClauses.map(emitter_1.emitHeritageClauses);
        const memberAssociations = (0, emitter_1.emitMemberAssociations)(d.memberAssociations);
        return [...classes, ...interfaces, ...enums, ...types, ...heritageClauses.flat(), ...memberAssociations];
    }).flat();
    if (entities.length === 0) {
        const errorMsg = "Could not process any class / interface / enum / type";
        console.log(chalk_1.default.red(errorMsg));
        entities.push(`[${errorMsg}]`);
    }
    return getStyling() + entities.join("\n");
}
function getStyling() {
    return '#.interface: fill=lightblue\n' +
        '#.enumeration: fill=lightgreen\n' +
        '#.type: fill=lightgray\n' +
        tsuml2_settings_1.SETTINGS.nomnoml.join("\n");
}
function createNomnomlSVG(settings) {
    // parse
    const declarations = parse(settings.tsconfig, settings.glob);
    if (declarations.length === 0) {
        console.log(chalk_1.default.red("\nno declarations found! tsconfig: " + settings.tsconfig, " glob: " + settings.glob));
        return;
    }
    // emit
    console.log(chalk_1.default.yellow("\nemitting declarations:"));
    const dsl = emit(declarations);
    const outDSL = settings.outDsl || tsuml2_settings_1.SETTINGS.outDsl;
    if (outDSL !== "") {
        console.log(chalk_1.default.green("\nwriting DSL"));
        fs.writeFile(outDSL, dsl, (err) => {
            if (err) {
                console.log(chalk_1.default.redBright("Error writing DSL file: " + err));
            }
        });
    }
    //render
    const outFile = settings.outFile || tsuml2_settings_1.SETTINGS.outFile;
    console.log(chalk_1.default.yellow("\nrender to svg"));
    let svg = (0, io_1.renderNomnomlSVG)(dsl);
    if (settings.typeLinks) {
        console.log(chalk_1.default.yellow("\nadding type links to svg"));
        svg = (0, emitter_1.postProcessSvg)(svg, outFile, declarations);
    }
    console.log(chalk_1.default.green("\nwriting SVG"));
    fs.writeFile(outFile, svg, (err) => {
        if (err) {
            console.log(chalk_1.default.redBright("Error writing file: " + err));
        }
    });
    return svg;
}
exports.createNomnomlSVG = createNomnomlSVG;
//# sourceMappingURL=index.js.map