"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.templates = void 0;
const typescript_1 = require("typescript");
const tsuml2_settings_1 = require("./tsuml2-settings");
exports.templates = {
    composition: "+->",
    implements: (interf, implementation) => {
        return (`${exports.templates.plainClassOrInterface(interf)}<:--${exports.templates.plainClassOrInterface(implementation)}`);
    },
    extends: (base, derived) => {
        return `${exports.templates.plainClassOrInterface(base)}<:-${exports.templates.plainClassOrInterface(derived)}`;
    },
    plainClassOrInterface: (name) => `[${name}]`,
    colorClass: (name) => `[${name}]`,
    colorInterface: (name) => `[${name}]`,
    class: (name, props, methods) => {
        return `[${name}|${props.map(propertyTemplate).join(";")}|${methods.map(methodTemplate).join(";")}]`;
    },
    interface: (name, props, methods) => {
        return `[<interface>${name}|${props.map(propertyTemplate).join(";")}|${methods.map(methodTemplate).join(";")}]`;
    },
    type: (name, props, methods) => {
        return `[<type>${name}|${props.map(propertyTemplate).join(";")}|${methods.map(methodTemplate).join(";")}]`;
    },
    enum: (name, enumItems) => {
        return `[<enumeration>${name}|${enumItems.join(";")}]`;
    },
    memberAssociation: memberAssociation
};
function methodTemplate(method) {
    // TODO go on
    let retVal = method.name + "()";
    if (method.returnType && tsuml2_settings_1.SETTINGS.propertyTypes) {
        retVal += ": " + method.returnType;
    }
    retVal = modifierTemplate(method.modifierFlags) + retVal;
    return retVal;
}
function propertyTemplate(property) {
    // TODO go on
    let retVal = property.name;
    if (property.type && tsuml2_settings_1.SETTINGS.propertyTypes) {
        if (property?.optional) {
            retVal += "?";
        }
        retVal += ": " + property.type;
    }
    retVal = modifierTemplate(property.modifierFlags) + retVal;
    return retVal;
}
function modifierTemplate(modifierFlags) {
    if (!tsuml2_settings_1.SETTINGS.modifiers) {
        return "";
    }
    let retVal = "";
    // UML2: static member should be underlined --> Not supported by nomnoml 
    if (modifierFlags & typescript_1.ModifierFlags.Static) {
        retVal = "static ";
    }
    if (modifierFlags & typescript_1.ModifierFlags.Private) {
        retVal = "-" + retVal;
    }
    else if (modifierFlags & typescript_1.ModifierFlags.Protected) {
        retVal = "\\#" + retVal;
    }
    else {
        retVal = "+" + retVal;
    }
    return retVal;
}
function memberAssociation(association) {
    return `${exports.templates.plainClassOrInterface(association.a.name)} ${association.a.multiplicity ?? ''} - ${association.b.multiplicity ?? ''} ${exports.templates.plainClassOrInterface(association.b.name)}`;
}
//# sourceMappingURL=templates.js.map