"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SETTINGS = exports.TsUML2Settings = void 0;
const fs = __importStar(require("fs"));
class TsUML2Settings {
    constructor() {
        /**
         * required
         */
        this.glob = "";
        /**
         * the path to the tsconfig.json file
         */
        this.tsconfig = "./tsconfig.json";
        /**
         * the svg output file (with relative or absolute path)
         */
        this.outFile = "out.svg";
        /**
         * show property types
         */
        this.propertyTypes = true;
        /**
         * show private, protected, public, static modifiers
         */
        this.modifiers = true;
        /**
         * add type links
         */
        this.typeLinks = true;
        /**
         * nomnoml layouting and styling options
         */
        this.nomnoml = [];
        /**
         * output file contain the DSL (nomnoml)
         */
        this.outDsl = "";
        /**
         * show associations between classes, interfaces, types and their member types
         */
        this.memberAssociations = false;
    }
    /**
     * parses a json file and merges in the provided options
     * @param json
     */
    formJSON(json) {
        Object.assign(this, JSON.parse(json));
    }
    fromArgs(yargs) {
        const argv = yargs.option('glob', {
            describe: "pattern to match the source files (i.e.: ./src/**/*.ts)",
            alias: "g",
            string: true,
            required: true
        }).option('tsconfig', {
            default: this.tsconfig,
            describe: "the path to tsconfig.json file"
        }).option('outFile', {
            describe: "the path to the output file",
            alias: "o",
            default: this.outFile,
        }).option('propertyTypes', {
            default: this.propertyTypes,
            describe: "show property types and method return types",
            boolean: true
        }).option('modifiers', {
            default: this.modifiers,
            describe: "show modifiers like public,protected,private,static",
            boolean: true
        }).option('typeLinks', {
            default: this.typeLinks,
            describe: "add links for classes, interface, enums that point to the source files",
            boolean: true
        }).option('nomnoml', {
            describe: "nomnoml layouting and styling options (an array of strings, each representing a nomnoml line), i.e.: --nomnoml \"#arrowSize: 1\" \"#.interface: fill=#8f8 dashed\" ",
            array: true,
            string: true
        }).option('outDsl', {
            describe: "the path to the output DSL file (nomnoml)",
            string: true,
            required: false,
        }).option('memberAssociations', {
            describe: "show associations between classes, interfaces, types and their member types",
            alias: 'm',
            boolean: true,
            required: false,
            default: this.memberAssociations
        }).option('config', {
            describe: "path to a json config file (command line options can be provided as keys in it)",
            string: true
        }).argv;
        if (argv.config) {
            //parse and apply the config file
            const config = fs.readFileSync(argv.config).toString();
            this.formJSON(config);
        }
        if (argv.glob) {
            this.glob = argv.glob;
        }
        if (argv.tsconfig && !yargs.parsed.defaulted.tsconfig) {
            this.tsconfig = argv.tsconfig;
        }
        if (argv.outFile && !yargs.parsed.defaulted.outFile) {
            this.outFile = argv.outFile;
        }
        if (argv.nomnoml) {
            this.nomnoml = argv.nomnoml;
        }
        if (argv.modifiers != null && !yargs.parsed.defaulted.modifiers) {
            this.modifiers = argv.modifiers;
        }
        if (argv.propertyTypes != null && !yargs.parsed.defaulted.propertyTypes) {
            this.propertyTypes = argv.propertyTypes;
        }
        if (argv.typeLinks != null && !yargs.parsed.defaulted.typeLinks) {
            this.typeLinks = argv.typeLinks;
        }
        if (argv.outDsl != null && !yargs.parsed.defaulted.outDsl) {
            this.outDsl = argv.outDsl;
        }
        if (argv.memberAssociations != null && !yargs.parsed.defaulted.memberAssociations) {
            this.memberAssociations = argv.memberAssociations;
        }
    }
}
exports.TsUML2Settings = TsUML2Settings;
/*
    function isUserSetOption(option: string,yargs: Argv<{}>) {
        function searchForOption(option:string) {
          return (process.argv.indexOf(option) > -1);
        }
    
        if (searchForOption(`-${option}`) || searchForOption(`--${option}`))
          return true;
    
        // Handle aliases for same option
        for (let aliasIndex in yargs.choices().parsed.aliases[option]) {
          let alias = yargs.choices().parsed.aliases[option][aliasIndex];
          if (searchForOption(`-${alias}`) || searchForOption(`--${alias}`))
            return true;
        }
    
        return false;
    }
*/
exports.SETTINGS = new TsUML2Settings();
//# sourceMappingURL=tsuml2-settings.js.map